
module.exports = apps;
apps.list = list;
apps.read = read;
apps.logs = logs;
apps.create = create;
apps.deleteApps = deleteApps;


apps.usage = "\nfhc apps "
  + "\nfhc apps (read) <app id>"
  + "\nfhc apps create <app title> (<git repo> <key file>)"
  + "\nfhc apps import <feedhenry zip file>"
  + "\nfhc apps logs <app id>"
  + "\nfhc apps delete <app id>"
  + "\nfhc apps search <query>";

var log = require("./utils/log");
var fhc = require("../fhc");
var fhreq = require("./utils/request");
var common = require("./common");
var util = require('util');
var async = require('async');
var path = require('path');
var fs = require('fs');
var url = require('url');
var https = require('https');
var exec = require("./utils/exec.js");

function apps (args, cb) { 
  if (args.length == 0){
    return list(cb);
  }else{
    if (args[0] == 'read') {
      var appId = args[1];
      return read(appId, cb);
    }else if (args[0] == 'create'){
      var title = args[1];
      var repo = args[2];
      var key = args[3];
      return create(title, repo, key, cb);
    }else if (args[0] == 'import'){
      var file = args[1];
      return importApp(file, cb);
    }else if (args[0] == 'delete'){
        args.shift();
        var appId = args;
        return deleteApps(appId, cb);
    }else if (args[0] == 'preview'){
        args.shift();
        var appId = args;
        return preview(appId, cb);
    }else if (args[0] == 'search'){
      var query = args[1];
      return search(query, cb);
    }else if (args[0] == 'logs'){
      var appId = args[1];
      return logs(appId, cb);
    }else if (args.length == 1){
      var appId = args[0];
      return read(appId, cb);
    }else{
      return cb(apps.usage);
    }
  }
};

function list(cb) {
  var payload ={};
  common.doApiCall(fhreq.getFeedHenryUrl(), "box/srv/1.1/ide/" + fhc.domain + "/app/list", payload, "Error reading app: ", cb);
};

function read (appId, cb) {  
  if (!appId) return cb("No appId specified! Usage:\n" + apps.usage);

  var payload = {payload:{guid: appId}};
  common.doApiCall(fhreq.getFeedHenryUrl(), "box/srv/1.1/ide/" + fhc.domain + "/app/read", payload, "Error reading app: ", cb);
};

function create(appTitle, repo, key, cb) {
  var params =  {
      title: appTitle,
      height: 300,
      width: 200,
      description: appTitle,
      config: {'nodejs.enabled' : true}
  };
  if (repo != null && repo != undefined) {
    params.scmurl = repo;
  }
  if (key != null && key != undefined) {
    if (!path.existsSync(key)) {
      return cb("Key file does not exist: " + key);
    }
    var buf = fs.readFileSync(key);
    params.scmkey = buf.toString();
  }

  common.doApiCall(fhreq.getFeedHenryUrl(), "box/srv/1.1/ide/" + fhc.domain + "/app/create", params, "Error creating new app: ", cb);
};

function deleteApp(appId, cb) {
  var payload = {payload:{confirmed:"true",guid:appId},context:{}};
  common.doApiCall(fhreq.getFeedHenryUrl(), "box/srv/1.1/ide/" + fhc.domain + "/app/delete", payload, "Error deleting app: ",cb);
}

function deleteApps(appId, cb) {  
  var appIds = [], ai, tempId, tempPayload;
  appIds = 'string' === typeof appId ? [appId] : appId;
  async.map(appIds, deleteApp, function(err, results) {
    cb(err, results);      
  });
}

function search(query, cb) { 
  var payload = {payload:{search:query},context:{}};   
  common.doApiCall(fhreq.getFeedHenryUrl(), "box/srv/1.1/ide/" + fhc.domain + "/app/search", payload, "Error searching for app: ", cb);
};

function logs (appId, cb) {  
  if (!appId) return cb("No appId specified! Usage:\n" + apps.usage);
  
  var payload = {payload:{guid: appId}};
  common.doApiCall(fhreq.getFeedHenryUrl(), "box/srv/1.1/ide/" + fhc.domain + "/app/logs", payload,"Error showing logs: ", cb);
};

function preview(appId, cb) {
// TODO WORK IN PROGRESS!!
  exec(fhc.config.get("browser"), [url], log.er(cb,
    "Failed to open "+url+" in a browser.  It could be that the\n"+
    "'browser' config is not set.  Try doing the following:\n"+
    "    fhc config set browser google-chrome\n"+
    "or:\n"+
    "    fhc config set browser lynx\n"));
}


function unknown (action, cb) {
  cb("Usage:\n" + apps.usage);
};

//
// TODO - Zip file upload this should be more generic, and possibly handled by request.js
//

function importApp(file, cb) {
  if (!path.existsSync(file)) {
    return cb("File doesn't exist: " + file);
  }
  
  log.silly(file, "app import");

  var boundary = Math.random();
  var post_data = [];

  post_data.push(new Buffer(EncodeFieldPart(boundary, 'type', 'feedhenry'), 'ascii'));
  post_data.push(new Buffer(EncodeFieldPart(boundary, 'location', file), 'ascii'));
  post_data.push(new Buffer(EncodeFilePart(boundary, 'application/zip', 'location', file), 'ascii'));

  var file_reader = fs.createReadStream(file, {encoding: 'binary'});
  var file_contents = '';
  file_reader.on('data', function(data){
    file_contents += data;
  });
  file_reader.on('end', function(){
    post_data.push(new Buffer(file_contents, 'binary'));
    post_data.push(new Buffer("\r\n--" + boundary + "--"), 'ascii');
    doUpload(post_data, boundary, cb);
  });
};

function EncodeFieldPart(boundary,name,value) {
    var return_part = "--" + boundary + "\r\n";
    return_part += "Content-Disposition: form-data; name=\"" + name + "\"\r\n\r\n";
    return_part += value + "\r\n";
    return return_part;
}

function EncodeFilePart(boundary,type,name,filename) {
    var return_part = "--" + boundary + "\r\n";
    return_part += "Content-Disposition: form-data; name=\"" + name + "\"; filename=\"" + filename + "\"\r\n";
    return_part += "Content-Type: " + type + "\r\n\r\n";
    return return_part;
}

function doUpload(post_data, boundary, cb) {

  var length = 0;

  for(var i = 0; i < post_data.length; i++) {
    length += post_data[i].length;
  }

  var post_options = {
    host: url.parse(fhreq.getFeedHenryUrl()).hostname,
    port: '443',
    path: '/box/srv/1.1/ide/apps/app/import?location=appanat.zip',
    method: 'POST',
    headers : {
        'Content-Type' : 'multipart/form-data; boundary=' + boundary,
        'Content-Length' : length,
        'Cookie' : "feedhenry=" + fhc.config.get("cookie") + ";"
    }
  };

  var post_request = https.request(post_options, function(response){
    response.setEncoding('utf8');
    var data = '';
    response.on('data', function(chunk){
      data = data + chunk;
    });
    response.on('end', function(){
      log.silly(data, 'app import');

      data = JSON.parse(data);
      if(data.status == 'pending') {
        return common.waitForJob(data.cacheKey, 0, cb);
      }else {
        return cb(null, data); 
      }
    });
    response.on('error', function(err) {
      return cb(err, null);
    });
  });

  for (var i = 0; i < post_data.length; i++) {
    post_request.write(post_data[i]);
  }
  post_request.end();
}


