// defaults, types, and shorthands.

var path = require("path")
  , stdio = process.binding("stdio")
  , url = require("url")
  , Stream = require("stream").Stream
  , semver = require("semver")
  , stableFamily = semver.parse(process.version)
  , os = require("os")
  , nopt = require("nopt")
  , log = require("./log");

nopt.typeDefs.semver = { type: semver, validate: validateSemver };

function validateSemver (data, k, val) {
  if (!semver.valid(val)) return false;
  data[k] = semver.valid(val);
};

nopt.invalidHandler = function (k, val, type, data) {
  log.warn(k + "=" + JSON.stringify(val), "invalid config");
};

if (!stableFamily || (+stableFamily[2] % 2)) stableFamily = null;
else stableFamily = stableFamily[1] + "." + stableFamily[2];

var defaults;
Object.defineProperty(exports, "defaults", {get: function () {
  if (defaults) return defaults;
  return defaults =
    { argv : []
    , bindist : stableFamily
        && ( stableFamily + "-"
           + "ares" + process.versions.ares + "-"
           + "ev" + process.versions.ev + "-"
           + "openssl" + process.versions.openssl + "-"
           + "v8" + process.versions.v8 + "-"
           + process.platform + "-"
           + (process.arch ? process.arch + "-" : "")
           + os.release() )

      // are there others?
    , browser : process.platform === "darwin" ? "open" : "google-chrome"
    , color : true
    , cookie: ""
    , cftarget: ""   // CloudFoundry settings..
    , cfuser: ""
    , cfpwd: ""
    , fhcluster: ""
    , editor : process.env.EDITOR || "vi"
    , force : false
    , global : false
    , globalconfig : path.resolve(process.execPath, "..", "..", "etc", "fhcrc")
    , logfd : stdio.stderrFD
    , loglevel : "warn"
    , filter : undefined
    , outfd : stdio.stdoutFD
    , parseable : false
    , prefix : path.join(process.execPath, "..", "..")
    , proxy : process.env.HTTP_PROXY || process.env.http_proxy || null
    , feedhenry : "https://apps.feedhenry.com/"
    , messaging : "https://aws102msg1.feedhenry.net/" // TODO NOT PUBLIC
    , dynofarm : "http://localhost:8000/" // ALSO NOT PUBLIC
    , nocache : false
    , tar : process.env.TAR || "tar"
    , tmp : (process.env.TMPDIR || "/tmp")
    , usage : false
    , user : "nobody"
    , username : ""
    , userconfig : path.resolve( process.env.HOME
                          , process.platform === "win32"
                            ? "fhc-config" : ".fhcrc")
    , usertargets : path.resolve( process.env.HOME
                          , process.platform === "win32"
                            ? "fhc-targets" : ".fhctargets")

    , version : false
    , viewer: "man"
    , _exit : true
    };
}});

exports.types = {
    argv : NaN
  , bindist : [String, null]
  , browser : String
  , cache : path
  , color : ["always", Boolean]
  , cookie : String
  , cftarget : String
  , cfuser : String
  , cfpwd : String
  , fhcluster : String
  , depth : Number
  , description : Boolean
  , dev : Boolean
  , editor : path
  , force : Boolean
  , global : Boolean
  , globalconfig : path
  , group : [String, Number]
  , logfd : [Number, Stream]
  , loglevel : ["silent","win","error","warn","info","verbose","silly"]
  , filter : String
  , nocache : Boolean
  , outfd : [Number, Stream]
  , parseable : Boolean
  , pre: Boolean
  , prefix: path
  , proxy : url
  , feedhenry : url
  , rollback : Boolean
  , shell : path
  , tar : String
  , tmp : path
  , "unsafe-perm" : Boolean
  , usage : Boolean
  , user : String
  , username : String
  , userconfig : path
  , version : Boolean
  , viewer: path
  , _exit : Boolean
};

exports.shorthands = {
    s : ["--loglevel", "silent"]
  , d : ["--loglevel", "info"]
  , dd : ["--loglevel", "verbose"]
  , ddd : ["--loglevel", "silly"]
  , feedhenry : ["--feedhenry"]
  , silent : ["--loglevel", "silent"]
  , verbose : ["--loglevel", "verbose"]
  , h : ["--usage"]
  , H : ["--usage"]
  , "?" : ["--usage"]
  , help : ["--usage"]
  , v : ["--version"]
  , f : ["--force"]
  , desc : ["--description"]
  , "n" : ["--nocache"]
  , "local" : ["--no-global"]
  , l : ["--long"]
  , p : ["--parseable"]
  , porcelain : ["--parseable"]
  , g : ["--global"]
};
